package java.awt.image;


public class DataBufferByte extends DataBuffer {

    private byte[] data;
    private byte[][] bankData;

    public DataBufferByte(int size) {
	super(TYPE_BYTE, size);
	data = new byte[size];
    }

    public DataBufferByte(int size, int numBanks) {
	super(TYPE_BYTE, size, numBanks);
	bankData = new byte[numBanks][size];
	data = bankData[0];
    }

    public DataBufferByte(byte[] dataArray, int size) {
	super(TYPE_BYTE, size);
	data = dataArray;
    }
    
    public DataBufferByte(byte[] dataArray, int size, int offset) {
	super(TYPE_BYTE, size, 1, offset);
	data = dataArray;
    }

    public DataBufferByte(byte[][] dataArray, int size) {
	super(TYPE_BYTE, size, dataArray.length);
	bankData = dataArray;
	data = bankData[0];
    }

    public DataBufferByte(byte[][] dataArray, int size, int[] offsets) {
	super(TYPE_BYTE, size, dataArray.length, offsets);
	bankData = dataArray;
	data = bankData[0];
    }

    public byte[] getData() {
	return data;
    }
    
    public byte[] getData(int bank) {
	return bankData[bank];
    }
    
    public byte[][] getBankData() {
	return bankData;
    }

    public int getElem(int i) {
	return data[i+offset] & 0xff; // get unsigned byte as int
    }

    public int getElem(int bank, int i) {
	// get unsigned byte as int
	return bankData[bank][i+offsets[bank]] & 0xff;
    }

    public void setElem(int i, int val) {
	data[i+offset] = (byte) val;
    }

    public void setElem(int bank, int i, int val) {
	bankData[bank][i+offsets[bank]] = (byte) val;
    }
}
