package no.rwr.image;

import java.awt.image.*;
import java.util.*;
import no.rwr.image.hermes.*;
import no.rwr.util.ArrayUtil;


public class Renderers implements RendererFactory {

    static final boolean DEBUG = false;
    static Renderers instance;

    public static Renderers getInstance() {
	if (instance == null) instance = new Renderers();
	return instance;
    }
    
    Vector factories = new Vector();
    
    public Renderers() {
	

	String[] factoryNames = {
	    "no.rwr.image.hermes.HermesFactory",
	    "no.rwr.image.BasicRendererFactory"
	};
	
	
	for (int i=0; i<factoryNames.length; i++) {
	    addFactory(factoryNames[i]);
	}
    }
	
    public void addFactory(String factoryName) {
	ClassLoader cl = ClassLoader.getSystemClassLoader();
	try {
	    
	    RendererFactory factory = (RendererFactory)
		cl.loadClass(factoryName).newInstance();
	    addFactory(factory);
	} catch (ClassNotFoundException e) {
	    if (DEBUG) System.err.println(e);
	} catch (InstantiationException e) {
	    if (DEBUG) System.err.println(e);
	} catch (IllegalAccessException e) {
	    if (DEBUG) System.err.println(e);
	}
    }
    

    public void addFactory(RendererFactory rf) {
	factories.addElement(rf);
    }
    
	
    

    public RasterOp createRenderer(ColorModel srcCM,
				   SampleModel srcSM,
				   ColorModel destCM,
				   SampleModel destSM) {


	Enumeration enum = factories.elements();
	while (enum.hasMoreElements()) {
	    RendererFactory factory = (RendererFactory) enum.nextElement();
	    try {
		return factory.createRenderer(srcCM, srcSM, destCM, destSM);
	    } catch (UnsupportedOperationException uoe) {
		if (DEBUG) System.err.println(uoe);;
	    }
	}

	String msg = "none of the registered factories can produce a " +
	    "to convert from CM=" + srcCM + ", SM=" + srcSM + ", to " +
	    "CM=" + destCM + ", SM=" + destSM;
	throw new UnsupportedOperationException(msg);
    }
    


}
