#! /bin/sh

# This was originally based on a one-liner from a paper by Peter Miller
# (It now does some serious mangeling of the gcc output.)

parsedopt=`getopt -o +h,o:,s: -l help,strip-prefix: -n $0 -- "$@"` || exit
eval set -- "$parsedopt"

# FIXME, use mktemp
tmp1=tmp1.tmp
tmp2=tmp2.tmp

while true ; do
    case "$1" in
    -h | --help)
        cat <<EOF
Usage: $0 [options] compiler-command...
Options:
  -h, --help            	print this message
  -o file			write dependency rule to <file>
  -s, --strip-prefix=directory	strip directory prefix from file references
EOF
        exit 0 ;;
    -o) outfile=$2; shift 2 ;;
    -s | --strip-prefix) stripprefix=$2 ; shift 2 ;;
    --) shift ; break ;;
    *) echo "$0: bad getopt: $*" ; exit 1 ;;

    esac
done

cmd="$*"
dependopt="-M -MG"


if $cmd $dependopt > $tmp1 ; then
    # OK
    true
else
    rm -f $tmp1 $tmp2
    exit 1
fi

sed < $tmp1 > $tmp2 \
    -e 's@ /[^ ]*@@g' -e 's@ ./@ @g' -e '/^ *\\/d'

tmpfile=$tmp2

if [ -n "$stripprefix" ] ; then
    sed < $tmp2 > $tmp1 \
	-e "s@^${stripprefix}/\?@@g" -e "s@ ${stripprefix}/\?@ @g"
    tmpfile=$tmp1
fi

if [ -n "$outfile" ] ; then
    echo "$outfile \\" > $outfile
    cat $tmpfile >> $outfile
else
    cat $tmpfile
fi

rm -f $tmp1 $tmp2